*$ CREATE SOURCE.FOR
*COPY SOURCE
*
*=== source ===========================================================*
*
      SUBROUTINE SOURCE ( NOMORE )

      INCLUDE '(DBLPRC)'
      INCLUDE '(DIMPAR)'
      INCLUDE '(IOUNIT)'
*
*----------------------------------------------------------------------*
*                                                                      *
*     Copyright (C) 1990-2010      by    Alfredo Ferrari & Paola Sala  *
*     All Rights Reserved.                                             *
*                                                                      *
*                                                                      *
*     New source for FLUKA9x-FLUKA20xy:                                *
*                                                                      *
*     Created on 07 January 1990   by    Alfredo Ferrari & Paola Sala  *
*                                                   Infn - Milan       *
*                                                                      *
*     Last change on  17-Oct-10    by    Alfredo Ferrari               *
*                                                                      *
*  This is just an example of a possible user written source routine.  *
*  note that the beam card still has some meaning - in the scoring the *
*  maximum momentum used in deciding the binning is taken from the     *
*  beam momentum.  Other beam card parameters are obsolete.            *
*                                                                      *
*       Output variables:                                              *
*                                                                      *
*              Nomore = if > 0 the run will be terminated              *
*                                                                      *
*----------------------------------------------------------------------*
*
      INCLUDE '(BEAMCM)'
      INCLUDE '(FHEAVY)'
      INCLUDE '(FLKSTK)'
      INCLUDE '(IOIOCM)'
      INCLUDE '(LTCLCM)'
      INCLUDE '(PAPROP)'
      INCLUDE '(SOURCM)'
      INCLUDE '(SUMCOU)'
*
      LOGICAL LFIRST
*
      SAVE LFIRST
      DATA LFIRST / .TRUE. /
*======================================================================*
*                                                                      *
*                 BASIC VERSION                                        *
*                                                                      *
*======================================================================*
      NOMORE = 0
*  +-------------------------------------------------------------------*
*  |  First call initializations:
      IF ( LFIRST ) THEN
         WRITE (LUNOUT,*) ' Seans custom SOURCE with Armstrong flux'

*  |  *** The following 3 cards are mandatory ***
         TKESUM = ZERZER
         LFIRST = .FALSE.
         LUSSRC = .TRUE.
*  |  *** User initialization ***
      END IF
*  |
*  +-------------------------------------------------------------------*
*  Push one source particle to the stack. Note that you could as well
*  push many but this way we reserve a maximum amount of space in the
*  stack for the secondaries to be generated
*  Npflka is the stack counter: of course any time source is called it
*  must be =0
      NPFLKA = NPFLKA + 1
      WRITE (LUNOUT,*) ' Fluka particle num',IJBEAM
*  Wt is the weight of the particle
*      WTFLK  (NPFLKA) = ONEONE
*      WEIPRI = WEIPRI + WTFLK (NPFLKA)
*  Particle type (1=proton.....). Ijbeam is the type set by the BEAM
*  card
*  +-------------------------------------------------------------------*
*  |  (Radioactive) isotope:
      IF ( IJBEAM .EQ. -2 .AND. LRDBEA ) THEN
         IARES  = IPROA
         IZRES  = IPROZ
         IISRES = IPROM
         CALL STISBM ( IARES, IZRES, IISRES )
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
*  |
*  +-------------------------------------------------------------------*
*  |  Heavy ion:
      ELSE IF ( IJBEAM .EQ. -2 ) THEN
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
         ILOFLK (NPFLKA) = IJHION
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |  Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
*  |
*  +-------------------------------------------------------------------*
*  |  Normal hadron:
      ELSE
         IONID = IJBEAM
         ILOFLK (NPFLKA) = IJBEAM
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |  Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
      END IF
*  |
*  +-------------------------------------------------------------------*
*  From this point .....
*  Particle generation (1 for primaries)
      LOFLK  (NPFLKA) = 1
*  User dependent flag:
      LOUSE  (NPFLKA) = 0
*  No channeling:
      LCHFLK (NPFLKA) = .FALSE.
      DCHFLK (NPFLKA) = ZERZER
*  User dependent spare variables:
      DO 100 ISPR = 1, MKBMX1
         SPAREK (ISPR,NPFLKA) = ZERZER
 100  CONTINUE
*  User dependent spare flags:
      DO 200 ISPR = 1, MKBMX2
         ISPARK (ISPR,NPFLKA) = 0
 200  CONTINUE
*  Save the track number of the stack particle:
      ISPARK (MKBMX2,NPFLKA) = NPFLKA
      NPARMA = NPARMA + 1
      NUMPAR (NPFLKA) = NPARMA
      NEVENT (NPFLKA) = 0
      DFNEAR (NPFLKA) = +ZERZER
*  ... to this point: don't change anything
*  Particle age (s)
      AGESTK (NPFLKA) = +ZERZER
      AKNSHR (NPFLKA) = -TWOTWO
* Now I am implementing ARMSTRONG FLUX
* Gen momentum to give E 10 MEV to 5 GeV
* Corresponds to P 141.77 MeV to 6000 MeV
*      E1=141.0D0
*      E2=6000.0D0
      E1=0.01D0
      E2=5.0D0
*      KENERGY=DBLE(KENERGY)
*      PBEAM=141.0D0+(E2-E1)*FLRNDM(DUMMY)
      BIGGS=0.01D0+(E2-E1)*DBLE(FLRNDM(DUMMY))
  
*      BIGGS=BIGGS/1000.0D0
      WRITE (LUNOUT,*) 'Random En=',DBLE(BIGGS)
*  Kinetic energy of the particle (GeV)
*      BIGGS= SQRT(PBEAM**2+ FLOAT(AM(IONID)**2 )) - FLOAT(AM(IONID))
*     WRITE (LUNOUT,*) 'Random En=',BIGGS
      TKEFLK (NPFLKA) = BIGGS
      WRITE (LUNOUT,*) 'Random En=',TKEFLK(NPFLKA)
*  Particle momentum
*      PMOFLK (NPFLKA) = PBEAM
      PMOFLK (NPFLKA) = SQRT ( TKEFLK (NPFLKA) * ( TKEFLK (NPFLKA)
     &                       + TWOTWO * AM (IONID) ) )
      WRITE (LUNOUT,*) 'Random P=',PMOFLK(NPFLKA)
* Calculate Weights
      PHIZ1=-4.7959D0
      PHIZ2=-5.301D0
      SLOP1=-0.50515D0
      SLOP2=-1.7696D0
      P2FAC=5.D-7
      PFAC2SL=-0.76956D0
      PFAC2INT=-6.301D0
      COSSPC=1.0D0
      WRITE (LUNOUT,*) ' Particle Type',IJBEAM
* Neutron
      IF ( IJBEAM .EQ. 8) THEN
         IF ( BIGGS .GE. 0.01D0 .AND. BIGGS .LE. 0.1D0) THEN
            COSSPC=SLOP1*LOG10(BIGGS*0.1D0*1000.0D0)+PHIZ1
            COSSPC=(10.0D0)**COSSPC
      
         ELSE IF( BIGGS .GT. 0.1D0) THEN
            COSSPC=SLOP2*LOG10(BIGGS*0.01D0*1000.0D0)+PHIZ2
            COSSPC=(10.0D0)**COSSPC

         END IF
* Proton
      ELSE IF ( IJBEAM .EQ. 1) THEN
         IF ( BIGGS .LE. 1.0D0 .AND. BIGGS .GT. 0.001D0) THEN
            COSSPC=PFAC2SL*LOG10(BIGGS*0.01D0*1000.0D0)+PFAC2INT
            COSSPC=(10.0D0)**COSSPC
*           COSSPC=1.0D0


         ELSE IF(BIGGS .GT. 1.0D0) THEN
            COSSPC=SLOP2*LOG10(BIGGS*0.01D0*1000.0D0)+PHIZ2
            COSSPC=(10.0D0)**COSSPC
*           COSSPC=1.0D0

*           COSSPC=COSSPC*10.0D-6
         END IF
      END IF
      COSSPC=(160.0D0*160.0D0)*2.0D0*(4.0D0)*ATAN(1.0D0)*COSSPC
      WRITE (LUNOUT,*) ' Weight',COSSPC
*      IF ( IJBEAM .EQ. -2 .AND. LRDBEA ) THEN
*   Wt is the weight of the particle
      WTFLK  (NPFLKA) = COSSPC
      WEIPRI = WEIPRI + WTFLK (NPFLKA)

      
*  Cosines (tx,ty,tz)
      TXFLK  (NPFLKA) = UBEAM
      TYFLK  (NPFLKA) = VBEAM
      TZFLK  (NPFLKA) = WBEAM
*     TZFLK  (NPFLKA) = SQRT ( ONEONE - TXFLK (NPFLKA)**2
*    &                       - TYFLK (NPFLKA)**2 )
*  Polarization cosines:
      TXPOL  (NPFLKA) = -TWOTWO
      TYPOL  (NPFLKA) = +ZERZER
      TZPOL  (NPFLKA) = +ZERZER
*  Particle coordinates
      XFLK   (NPFLKA) = XBEAM
      YFLK   (NPFLKA) = YBEAM
      ZFLK   (NPFLKA) = ZBEAM
*  Calculate the total kinetic energy of the primaries: don't change
      IF ( ILOFLK (NPFLKA) .EQ. -2 .OR. ILOFLK (NPFLKA) .GT. 100000 )
     &   THEN
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      ELSE IF ( ILOFLK (NPFLKA) .NE. 0 ) THEN
         TKESUM = TKESUM + ( TKEFLK (NPFLKA) + AMDISC (ILOFLK(NPFLKA)) )
     &          * WTFLK (NPFLKA)
      ELSE
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      END IF
      RADDLY (NPFLKA) = ZERZER
*  Here we ask for the region number of the hitting point.
*     NREG (NPFLKA) = ...
*  The following line makes the starting region search much more
*  robust if particles are starting very close to a boundary:
      CALL GEOCRS ( TXFLK (NPFLKA), TYFLK (NPFLKA), TZFLK (NPFLKA) )
      CALL GEOREG ( XFLK  (NPFLKA), YFLK  (NPFLKA), ZFLK  (NPFLKA),
     &              NRGFLK(NPFLKA), IDISC )
*  Do not change these cards:
      CALL GEOHSM ( NHSPNT (NPFLKA), 1, -11, MLATTC )
      NLATTC (NPFLKA) = MLATTC
      CMPATH (NPFLKA) = ZERZER
      CALL SOEVSV
      RETURN
*=== End of subroutine Source =========================================*
      END

